@@ -1,2 +1,2 @@
 from .confusion import calculate_direct_confusion
-from .seurat import find_anchor, integrate
+from .seurat_class import SeuratIntegration
