@@ -10,6 +10,7 @@ import numpy as np
 import pandas as pd
 import xarray as xr
 import yaml
+import scipy.sparse as ss
 from pybedtools import BedTool
 
 from .utilities import (
@@ -312,8 +313,33 @@ class MCDS(xr.Dataset):
 
         if chunks is not None:
             if chunks == 'auto':
-                chunks = {k: min(4096, max(v // 5, 1)) for k, v in ds.dims.items()}
-            ds = ds.chunk(chunks=chunks)
+                n_obs = ds.get_index(obs_dim).size
+                data_chunks = {}
+                for name, da in ds.data_vars.items():
+                    if 'chunks' in da.encoding:
+                        # if encoding exist, use the chunks in encoding
+                        data_chunks[name] = da.encoding['chunks']
+                    else:
+                        # else, use the default chunk setting
+                        chunks = {obs_dim: max(min(1000, max(n_obs // 10, 1)), 10),
+                                  # by default, no chunk on features
+                                  'mc_type': 1,
+                                  'count_type': 1}
+                        data_chunks[name] = tuple(chunks[k]
+                                                  if k in chunks else da.get_index(k).size
+                                                  for k in da.dims)
+                for name, da in ds.data_vars.items():
+                    if da.chunks is None:
+                        ds[name] = da.chunk(data_chunks[name])
+            else:
+                # user provide specific chunks
+                if isinstance(chunks, dict):
+                    _chunks = {dim: chunks[dim]
+                    if dim in chunks else ds.get_index(dim).size
+                               for dim in ds.dims}
+                    ds = ds.chunk(chunks=_chunks)
+                else:
+                    raise ValueError(f'chunks must be a dict or "auto", got {chunks}')
 
         return cls(ds, obs_dim=obs_dim, var_dim=var_dim).squeeze()
 
@@ -516,16 +542,16 @@ class MCDS(xr.Dataset):
             try:
                 black_feature_index = (
                     black_feature.to_dataframe()
-                        .set_index(["chrom", "start", "end"])
-                        .index
+                    .set_index(["chrom", "start", "end"])
+                    .index
                 )
             except pd.errors.EmptyDataError:
                 print("No feature overlapping the black list bed file.")
                 return self
         black_feature_id = pd.Index(
             feature_bed_df.reset_index()
-                .set_index(["chrom", "start", "end"])
-                .loc[black_feature_index][var_dim]
+            .set_index(["chrom", "start", "end"])
+            .loc[black_feature_index][var_dim]
         )
 
         print(
@@ -536,7 +562,7 @@ class MCDS(xr.Dataset):
             mcds = self.sel({var_dim: ~self.get_index(var_dim).isin(black_feature_id)})
         return mcds
 
-    def remove_chromosome(self, exclude_chromosome, var_dim=None):
+    def remove_chromosome(self, exclude_chromosome=None, include_chromosome=None, var_dim=None):
         """
         Remove regions in specific chromosome
 
@@ -545,18 +571,35 @@ class MCDS(xr.Dataset):
         var_dim
             Name of var_dim
         exclude_chromosome
-            Chromosome to remove
+            if provided, only these chromosomes will be removed
+        include_chromosome
+            if provided, only these chromosomes will be kept
         Returns
         -------
         MCDS (xr.Dataset)
         """
         var_dim = self._verify_dim(var_dim, mode='var')
-        judge = self.coords[f"{var_dim}_chrom"].isin(exclude_chromosome)
-        print(f"{int(judge.sum())} {var_dim} features in {exclude_chromosome} removed.")
-        with dask.config.set(**{"array.slicing.split_large_chunks": False}):
-            mcds = self.sel({var_dim: ~judge})
+        if exclude_chromosome is not None:
+            judge = self.coords[f"{var_dim}_chrom"].isin(exclude_chromosome)
+            print(f"{int(judge.sum())} {var_dim} features in {exclude_chromosome} removed.")
+            with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+                mcds = self.sel({var_dim: ~judge})
+        if include_chromosome is not None:
+            judge = self.coords[f"{var_dim}_chrom"].isin(include_chromosome)
+            print(f"{int(judge.sum())} {var_dim} features in {include_chromosome} kept.")
+            with dask.config.set(**{"array.slicing.split_large_chunks": False}):
+                mcds = self.sel({var_dim: judge})
         return mcds
 
+    def _get_da_name(self, var_dim, da_suffix):
+        name1 = f"{var_dim}_{da_suffix}"
+        if name1 in self:
+            return name1
+        name2 = f"{var_dim}_da_{da_suffix}"
+        if name2 in self:
+            return name2
+        raise KeyError(f"{name1} or {name2} not found in MCDS.")
+
     def calculate_hvf_svr(
             self,
             mc_type=None,
@@ -564,14 +607,19 @@ class MCDS(xr.Dataset):
             obs_dim=None,
             n_top_feature=5000,
             da_suffix="frac",
-            plot=True,
+            da_name=None,
+            plot=True
     ):
         from sklearn.svm import SVR
         import plotly.graph_objects as go
         obs_dim = self._verify_dim(obs_dim, mode='obs')
         var_dim = self._verify_dim(var_dim, mode='var')
 
-        frac_da = self[f"{var_dim}_da_{da_suffix}"]
+        if da_name is None:
+            frac_da_name = self._get_da_name(var_dim, da_suffix)
+        else:
+            frac_da_name = da_name
+        frac_da = self[frac_da_name]
         count_da = self[f"{var_dim}_da"]
         if 'mc_type' in self.dims:
             if mc_type is None:
@@ -654,8 +702,8 @@ class MCDS(xr.Dataset):
                         marker=dict(
                             size=2,
                             color=plot_data["feature_select"]
-                                .map({True: "red", False: "gray"})
-                                .tolist(),  # set color to an array/list of desired values
+                            .map({True: "red", False: "gray"})
+                            .tolist(),  # set color to an array/list of desired values
                             opacity=0.8,
                         ),
                     )
@@ -692,7 +740,7 @@ class MCDS(xr.Dataset):
             mean_binsize=0.05,
             cov_binsize=100,
             da_suffix="frac",
-            plot=True,
+            plot=True
     ):
         """
         Calculate normalized dispersion to select highly variable features.
@@ -723,6 +771,8 @@ class MCDS(xr.Dataset):
             bin size to separate features across mean
         cov_binsize
             bin size to separate features across coverage
+        da_suffix
+            Suffix to add to the name of the dataarray
         plot
             If true, will plot mean, coverage and normalized dispersion scatter plots.
 
@@ -733,7 +783,8 @@ class MCDS(xr.Dataset):
         obs_dim = self._verify_dim(obs_dim, mode='obs')
         var_dim = self._verify_dim(var_dim, mode='var')
 
-        frac_da = self[f"{var_dim}_da_{da_suffix}"]
+        frac_da_name = self._get_da_name(var_dim, da_suffix)
+        frac_da = self[frac_da_name]
         if 'mc_type' in self.dims:
             if mc_type is None:
                 mc_types = self.get_index('mc_type')
@@ -786,11 +837,69 @@ class MCDS(xr.Dataset):
                 self.coords[f"{var_dim}_{mc_type}_{name}"] = column
         return hvf_df
 
-    def get_score_adata(self, mc_type, quant_type, obs_dim=None, var_dim=None, sparse=True):
+    def get_count_adata(self,
+                        da_name,
+                        obs_dim=None,
+                        var_dim=None,
+                        sparse=True,
+                        loading_chunk=10000,
+                        binarize_cutoff=None,
+                        use_vars=None,
+                        split_large_chunks=False):
+        with dask.config.set(**{'array.slicing.split_large_chunks': split_large_chunks}):
+            obs_dim = self._verify_dim(obs_dim, mode='obs')
+            var_dim = self._verify_dim(var_dim, mode='var')
+            da = self[da_name].transpose(obs_dim, var_dim)
+
+            # load matrix by chunks
+            da_obs = da.get_index(obs_dim)
+            total_data = []
+            for chunk_start in range(0, da_obs.size, loading_chunk):
+                chunk_cells = da_obs[chunk_start:chunk_start + loading_chunk]
+                chunk_da = da.sel({obs_dim: chunk_cells}).load()
+
+                if use_vars is not None:
+                    chunk_da = chunk_da.sel({var_dim: use_vars})
+
+                if sparse:
+                    chunk = ss.csr_matrix(chunk_da.values)
+                else:
+                    chunk = chunk_da.values
+
+                # binarize the matrix to further reduce memory size
+                if binarize_cutoff is not None:
+                    chunk = (chunk > binarize_cutoff).astype(np.int8)
+                total_data.append(chunk)
+                print(f"Loading chunk {chunk_start}-{min(chunk_start + loading_chunk, da_obs.size)}"
+                      f"/{da_obs.size}")
+
+        # concatenate all chunks
+        if sparse:
+            total_data = ss.vstack(total_data)
+        else:
+            total_data = np.vstack(total_data)
+
+        obs_df, var_df = make_obs_df_var_df(da, obs_dim, var_dim)
+        adata = anndata.AnnData(X=total_data, obs=obs_df, var=var_df.loc[use_vars, :])
+
+        if 'chrom' in adata.var:
+            chroms = adata.var['chrom'].astype('category')
+            from natsort import natsorted
+            adata.var['chrom'] = chroms.cat.reorder_categories(natsorted(chroms.cat.categories))
+        return adata
+
+    def get_score_adata(self,
+                        mc_type,
+                        quant_type,
+                        obs_dim=None,
+                        var_dim=None,
+                        sparse=True,
+                        loading_chunk=50000,
+                        binarize_cutoff=None):
         QUANT_TYPES = ['hypo-score', 'hyper-score']
-        if quant_type.lower() == 'hypo':
+        if quant_type.lower().startswith('hypo'):
             quant_type = 'hypo-score'
-        elif quant_type.lower() == 'hyper':
+        elif quant_type.lower().startswith('hyper'):
             quant_type = 'hyper_score'
         else:
             pass
@@ -799,18 +908,13 @@ class MCDS(xr.Dataset):
 
         obs_dim = self._verify_dim(obs_dim, mode='obs')
         var_dim = self._verify_dim(var_dim, mode='var')
-
-        da = self[f'{var_dim}_da_{mc_type}-{quant_type}']
-        # TODO load by chunk and parallel to make IO faster
-        if sparse:
-            from scipy.sparse import csr_matrix
-            data = csr_matrix(da.transpose(obs_dim, var_dim).values)
-        else:
-            data = da.transpose(obs_dim, var_dim).values
-        # TODO validate chrom coords
-
-        obs_df, var_df = make_obs_df_var_df(da, obs_dim, var_dim)
-        adata = anndata.AnnData(X=data, obs=obs_df, var=var_df)
+        da_name = f'{var_dim}_da_{mc_type}-{quant_type}'
+        adata = self.get_count_adata(da_name=da_name,
+                                     obs_dim=obs_dim,
+                                     var_dim=var_dim,
+                                     sparse=sparse,
+                                     loading_chunk=loading_chunk,
+                                     binarize_cutoff=binarize_cutoff)
         return adata
 
     def get_adata(
@@ -820,7 +924,7 @@ class MCDS(xr.Dataset):
             var_dim=None,
             da_suffix="frac",
             select_hvf=True,
-            split_large_chunks=True,
+            split_large_chunks=True
     ):
         """
         Get anndata from MCDS mC rate matrix
@@ -860,15 +964,16 @@ class MCDS(xr.Dataset):
         with dask.config.set(
                 **{"array.slicing.split_large_chunks": split_large_chunks}
         ):
-            frac_da = self[f"{var_dim}_da_{da_suffix}"]
+            frac_da_name = self._get_da_name(var_dim, da_suffix)
+            frac_da = self[frac_da_name]
             if mc_type is None:
                 if select_hvf:
                     try:
                         use_features = (
                             self.coords[f"{var_dim}_feature_select"]
-                                .to_pandas()
-                                .dropna()
-                                .astype(bool)
+                            .to_pandas()
+                            .dropna()
+                            .astype(bool)
                         )
                         use_features = use_features[use_features].index
                         use_data = frac_da.sel({var_dim: use_features}).squeeze()
@@ -885,9 +990,9 @@ class MCDS(xr.Dataset):
                     try:
                         use_features = (
                             self.coords[f"{var_dim}_{mc_type}_feature_select"]
-                                .to_pandas()
-                                .dropna()
-                                .astype(bool)
+                            .to_pandas()
+                            .dropna()
+                            .astype(bool)
                         )
                         use_features = use_features[use_features].index
                         use_data = frac_da.sel({"mc_type": mc_type, var_dim: use_features}).squeeze()
@@ -913,7 +1018,7 @@ class MCDS(xr.Dataset):
             obs_dim=None,
             add_mc_frac=True,
             add_overall_mc=True,
-            overall_mc_da="chrom100k_da",
+            overall_mc_da="chrom100k_da"
     ):
         obs_dim = self._verify_dim(obs_dim, mode='obs')
 
@@ -938,13 +1043,13 @@ class MCDS(xr.Dataset):
                         overall_mc_dim = overall_mc_da[:-3]
                         mc = (
                             cluster_mcds[overall_mc_da]
-                                .sel(mc_type=mc_type, count_type="mc")
-                                .sum(dim=overall_mc_dim)
+                            .sel(mc_type=mc_type, count_type="mc")
+                            .sum(dim=overall_mc_dim)
                         )
                         cov = (
                             cluster_mcds[overall_mc_da]
-                                .sel(mc_type=mc_type, count_type="cov")
-                                .sum(dim=overall_mc_dim)
+                            .sel(mc_type=mc_type, count_type="cov")
+                            .sum(dim=overall_mc_dim)
                         )
                         cluster_mcds.coords[f"{cluster_col}_{mc_type}_overall"] = mc / cov
                 else:
@@ -962,13 +1067,12 @@ class MCDS(xr.Dataset):
         _region_ds = RegionDS(self, region_dim=region_dim)
         return _region_ds
 
-    def write_dataset(self,
-                      output_path,
+    def write_dataset(self, output_path,
                       mode='w-',
                       obs_dim=None,
                       var_dims: Union[str, list] = None,
                       use_obs=None,
-                      chunk_size=1000):
+                      chunks='auto'):
         """
         Write MCDS into an on-disk zarr dataset.
         Data arrays for each var_dim will be saved in separate sub-directories of output_path.
@@ -987,13 +1091,29 @@ class MCDS(xr.Dataset):
             dimension name, or a list of dimension names of variables
         use_obs
             Select and order observations according to this parameter when wrote to output_path.
-        chunk_size
-            The load and write chunks, set this as large as possible based on available memory.
+        chunks
+            Zarr chunks on disk and in memory.
 
         Returns
         -------
         output_path
         """
+        with dask.config.set(**{'array.slicing.split_large_chunks': False}):
+            self._write_dataset(output_path=output_path,
+                                mode=mode,
+                                obs_dim=obs_dim,
+                                var_dims=var_dims,
+                                use_obs=use_obs,
+                                chunks=chunks)
+        return
+
+    def _write_dataset(self,
+                       output_path,
+                       mode='w-',
+                       obs_dim=None,
+                       var_dims: Union[str, list] = None,
+                       use_obs=None,
+                       chunks='auto'):
         # save mcds to a location, if the location is a MCDS dataset, auto save to sub_dir and modify .ALLCools
         # deal with large save issue, load by chunk and write append
         obs_dim = self._verify_dim(obs_dim, mode='obs')
@@ -1037,6 +1157,19 @@ class MCDS(xr.Dataset):
                 raise IndexError(f'Not all obs id in use_obs parameter exist in MCDS '
                                  f'{self.obs_dim if obs_dim is None else obs_dim} dim')
 
+        # determine chunks
+        n_obs = obs_index.size
+        if (chunks == 'auto') or (chunks is None):
+            chunks = {obs_dim: max(min(1000, max(n_obs // 10, 1)), 10),
+                      'mc_type': 1,
+                      'count_type': 1}
+        full_chunks = {
+            dim: size
+            if ((dim not in chunks) or (chunks[dim] is None))
+            else chunks[dim]
+            for dim, size in self.dims.items()
+        }
+
         for var_dim, da_names in data_vars_dict.items():
             print(f'Saving {var_dim}')
             _this_output_path = f'{output_path}/{var_dim}'
@@ -1052,16 +1185,33 @@ class MCDS(xr.Dataset):
                 _mcds.attrs.pop('var_dim')
 
                 total_obs = obs_index.size if use_obs is None else use_obs.size
-                for i, chunk_start in enumerate(range(0, total_obs, chunk_size)):
+                obs_chunk_size = chunks[obs_dim]
+                for i, chunk_start in enumerate(range(0, total_obs, obs_chunk_size)):
+                    print(f'Saving chunk {i}: {chunk_start} - '
+                          f'{min(chunk_start + obs_chunk_size, total_obs)}')
                     if use_obs is None:
-                        _obs_chunk = obs_index[chunk_start:chunk_start + chunk_size]
+                        _obs_chunk = obs_index[chunk_start:chunk_start + obs_chunk_size]
                         _chunk_ds = _mcds.sel({obs_dim: _obs_chunk}).load()
                     else:
-                        _obs_chunk = use_obs[chunk_start:chunk_start + chunk_size]
+                        _obs_chunk = use_obs[chunk_start:chunk_start + obs_chunk_size]
                         # load first, order next, to prevent unordered index causing more chunks
                         _chunk_ds = _mcds.sel({obs_dim: obs_index.isin(_obs_chunk)}).load()
                         _chunk_ds = _chunk_ds.sel({obs_dim: _obs_chunk})
 
+                    # add chunk to encoding
+                    for var in _chunk_ds.variables:
+                        _chunk_ds[var].encoding['chunks'] = tuple(full_chunks[d]
+                                                                  for d in _chunk_ds[var].dims)
+                        if 'preferred_chunks' in _chunk_ds[var].encoding:
+                            _chunk_ds[var].encoding['preferred_chunks'] = {
+                                d: full_chunks[d]
+                                for d in _chunk_ds[var].dims
+                            }
+                        if 'compressor' in _chunk_ds[var].encoding:
+                            try:
+                                _chunk_ds[var].encoding['compressor'].clevel = 1
+                            except AttributeError:
+                                pass
                     if i == 0:
                         _chunk_ds.to_zarr(_this_output_path, mode='w')
                     else:
